# Example 2.19 of Mehttalo, L. and Lappi, J. 2020. Biometry for forestry and environmental data: With examples in R. CRC Press, 
# 
# Author: Lauri Mehttalo
###############################################################################

# Illustration of two stationary, isotropic gaussian spatial process 
# Generate a 10 by 10 lattice.
# x1 and x2 will include the coordinates of them.
x1<-rep(seq(10, 190, 20),10)
x2<-rep(seq(10, 190, 20),each=10)

# The expected value of Y for all locations i will be E(Y_i)=mu_i=6 and 
# variance will be var(Y_i)=1.5^2
mu<-rep(6,100)
var<-1.5^2

# To produce a variance-covariance matrix Sigma for the multivariate 
# (100-variate) normal distribution, let us first compute the 
# distances between all pairs of points. 
# This produces a 100 by 100 matrix of distances.
# The diagonal elements (distance from a point to itself)
# are naturally 0.
dist<-matrix(NA,ncol=100,nrow=100)
for (i in 1:100) {
     dist[i,]<-sqrt((x1[i]-x1)^2+(x2[i]-x2)^2)
     }

# Assume that the true correlation between each pair 
# is expressed by the powered exponential function
# rho(u)=exp(-(u/phi)^kappa)
# where phi and kappa defines the spatial scale of the correlation.
d<-seq(0,40,0.1)
corA<-exp(-(d/10)^3)
corB<-exp(-(d/10)^2)
corC<-exp(-(d/20)^2)

windows(width=7,height=7)
par(mfcol=c(2,2),mai=c(0.6,0.5,0.3,0.1),mgp=c(2,0.7,0))

set.seed(234567)
plot(d,corA,type="l",xlab="Distance",ylab="Correlation")
lines(d,corB,lty="dashed")
lines(d,corC,lwd=2)
legend("topright",legend=c(expression(kappa*"=3, " * phi==10),
                           expression(kappa*"=2, " * phi==10),
                           expression(kappa*"=2, " * phi==20)),
       lty=c("solid","dashed","solid"),lwd=c(1,1,2))

# We see that
# a) correlation is 1 when distance is 0
# b) correlation vanishes the faster the smaller the value of phi.

# We need MASS to generate realizations of a multinormal random vector
# using function mvrnorm
library(MASS)

# Define the variance-covariance matrices of Y, by assuming the 
# correlations specified above by options A, B and C.
# Because all Y's are assumed to have equal variance (1.5^2)
# cov(X,Y)=corr(X,Y)*sd(X)*sd(Y)=corr(X,Y)*var(Y).
Sigma<-var*exp(-(dist/20)^2)
y<-mvrnorm(1,mu,Sigma)

plot(x1,x2,cex=0.1,xlim=c(0,200),ylim=c(0,200),main="Realization 1")
symbols(x1,x2,circles=y,add=T,inches=F)

# Produce two more realizations
for (i in 1:2){
     plot(x1,x2,cex=0.1,xlim=c(0,200),ylim=c(0,200),main=paste("Realization",i+1))
     symbols(x1,x2,circles=mvrnorm(1,mu,Sigma),add=T,inches=F)
     }
